#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if !( defined(__EXTENSIONS__) || defined( __STDC__ ) && __STDC__ == 0 || defined(_XOPEN_SOURCE) )
 #include "getopt.h"
#endif

void Help( const char *pccSelfName )
{
 printf( "\n" );
 printf( "2004(C)SVKsoft \"File Cuter\"\n" );
 printf( "%s [-i iparam] [-o oparam] [-f fparam] [-l lparam] [-t tparam] [-e eparam] [-h]\n", pccSelfName );
 printf( "\n" );
 printf( "\t-i <Input filename>         (default: stdin)\n" );
 printf( "\t-o <Output filename>        (default: stdout)\n" );
 printf( "\t-f <Pointer from>           (default: 0)\n" );
 printf( "\t-l <Length after from>      (default: up to end)\n" );
 printf( "\t-t <Pointer after from>     (default: up to end)\n" );
 printf( "\t-e <Length before end>      (default: 0)\n" );
 printf( "\t-b <Length from before end> (default: from begin)\n" );
 printf( "\t-a Append to output file    (default: create new)\n" );
 printf( "\t-h This help information\n" );
}

int main(int argc, char* argv[])
{
    extern char *optarg;
    int iKey;
    fpos_t iFrom = 0, iLen = 0, iTo = 0, iExclude = 0, iCur = 0, iBefEnd = 0;
    char *pcIn = 0, *pcOut = 0, cOk = 1, cAppend = 0;
    unsigned char ucChr;
    FILE *poIn, *poOut;
    fpos_t oPos;

    while ( ( iKey = getopt( argc, argv, "i:o:f:l:t:e:b:ha" ) ) != EOF )
	{
     switch( iKey )
     {
      case 'i': pcIn     = optarg; break; //Input filename ( def - stdin )
      case 'o': pcOut    = optarg; break; //Output filename ( def - stdout )
      case 'f': iFrom    = atoi( optarg ); break;//Pointer from ( def - 0 )
      case 'l': iLen     = atoi( optarg ); break;//Length after from ( def - up to end )
      case 't': iTo      = atoi( optarg ); break;//Pointer after from ( def - up to end )
      case 'e': iExclude = atoi( optarg ); break;//Length before end ( def - 0 )
      case 'b': iBefEnd  = atoi( optarg ); break;//Length from before end ( def - from begin )
      case 'a': cAppend  = 1; break;//Append to output file
      case 'h':
      default : Help( argv[ 0 ] );
                cOk      = 0; break;
     }
    }

    if( cOk )
    do
    {
     cOk = 0;

     if( pcIn )
     {
      if( !( poIn = fopen( pcIn, "rb" ) ) ) break;
     }
      else poIn = stdin;

     if( pcOut )
     {
      if( !cAppend && !( poOut = fopen( pcOut, "wb" ) ) ||
           cAppend && !( poOut = fopen( pcOut, "ab" ) )
        ) break;
     }
      else poOut = stdout;

     if( pcIn )
     {
      if( fseek( poIn, 0, SEEK_END ) != 0 ||
          fgetpos( poIn, &oPos ) != 0 ||
          fseek( poIn, 0, SEEK_SET ) != 0
        ) break;

      if( iBefEnd > oPos ||
          iFrom && iBefEnd &&
          iFrom + iBefEnd != oPos
        ) break;

      if( !iFrom && iBefEnd )
       iFrom = oPos - iBefEnd;
     }

     if( iLen && iTo &&
         iFrom + iLen != iTo
       ) break;

     if( !iLen && iTo )
      iLen = iTo - iFrom;

     if( !pcIn )
     {
      if( iExclude || iBefEnd ) break;
     }
     else
     {
      if( iExclude )
      {
       if( iExclude > oPos ) break;

       if( iLen )
       {
        if( iFrom + iLen + iExclude != oPos ) break;
       }
       else
        iLen = oPos - iExclude - iFrom;
      }
     }

     if( pcIn && pcOut && strcmp( pcIn, pcOut ) == 0 )
      break;

     cOk = 1;
    }while( 0 );

    while( cOk )
    {
     if( fread( &ucChr, sizeof( ucChr ), 1, poIn ) == 1 )
     {
      iCur ++;
      if( iCur > iFrom )
      {
       if( !iLen || iCur <= iFrom + iLen )
       {
        if( fwrite( &ucChr, sizeof( ucChr ), 1, poOut ) != 1  )
         cOk = 0;
       }
      }
     }
     else
     {
      if( iLen && iCur <= iFrom + iLen )
       cOk = 0;
      else
       break;
     }
    }

    if( cOk ) return 0;
    else      return -1;
}
